"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConfigService = void 0;
const ramda_1 = require("ramda");
const class_validator_1 = require("class-validator");
const class_transformer_1 = require("class-transformer");
const dotenv_1 = require("dotenv");
const exceptions_1 = require("./exceptions");
const errors_1 = require("./errors");
class ConfigService {
    options;
    configMap;
    constructor(provides, options = {}) {
        this.options = options;
        this.configMap = this.getConfigMap(provides);
    }
    get(config) {
        if (!this.configMap.has(config)) {
            throw new exceptions_1.ConfigServiceException(`${errors_1.ConfigServiceError.CONFIG_NOT_FOUND}: [class ${config.name}]`);
        }
        return this.configMap.get(config);
    }
    getConfigEntry(config) {
        const { parsed } = (0, dotenv_1.config)();
        const { overrides } = this.options;
        const { options: { transformOptions } } = this.options.parent || this;
        const keys = Object.keys(new config());
        const env = (0, ramda_1.pickAll)(keys, overrides || (0, ramda_1.mergeAll)([{}, process.env, parsed]));
        return (0, class_transformer_1.plainToInstance)(config, env, {
            enableImplicitConversion: true,
            exposeDefaultValues: true,
            ...transformOptions
        });
    }
    getConfigEntries(config) {
        return config.map(constructor => [constructor, this.getConfigEntry(constructor)]);
    }
    getConfigMap(config) {
        const { parent } = this.options;
        const entries = this.getConfigEntries(config);
        if (parent) {
            const configsWithBase = [...parent.configMap.entries(), ...entries];
            return this.validate(new Map(configsWithBase));
        }
        return this.validate(new Map(entries));
    }
    validate(configMap) {
        const { options: { validatorOptions } } = this.options.parent || this;
        const configs = [...configMap.values()];
        const validationErrors = configs.reduce((errors, config) => {
            const validationErrors = (0, class_validator_1.validateSync)(config, validatorOptions);
            if (!validationErrors.length) {
                return errors;
            }
            return [...errors, { config, validationErrors }];
        }, []);
        if (validationErrors.length) {
            throw new exceptions_1.ConfigServiceException(errors_1.ConfigServiceError.CONFIG_VALIDATION_ERRORS, {
                stack: validationErrors.reduce((formattedStack, { config, validationErrors }) => {
                    formattedStack += `Failed to validate config: [class ${config.constructor.name}]\r\n`;
                    formattedStack += validationErrors
                        .map(error => Object.entries(error.constraints || {}).map(([_, message]) => `\x1b[31m└─ \x1b[90m${message} (found: ${error.value})\x1b[0m`))
                        .join('\r\n');
                    return formattedStack;
                }, '')
            });
        }
        return configMap;
    }
}
exports.ConfigService = ConfigService;
//# sourceMappingURL=config.service.js.map